<?php

namespace App\Controller;

use App\Entity\Snip;
use App\Form\SnipType;
use App\Repository\SnipRepository;
use App\Security\Voter\SnipVoter;
use App\Service\SnipServiceFactory;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

#[Route('/snip', name: 'snip')]
class SnipController extends AbstractController
{
    public function __construct(
        private readonly SnipRepository     $repository,
        private readonly SnipServiceFactory $snipServiceFactory,
    )
    {
    }

    #[Route('/', name: '_index')]
    public function index(): Response
    {
        return $this->render('snip/index.html.twig', [
            'snips' => $this->repository->findByUser($this->getUser()),
        ]);
    }

    #[Route('/single/{snip}', name: '_single')]
    public function single(Snip $snip): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::VIEW, $snip);

        return $this->render('snip/single.html.twig', [
            'snip' => $snip,
            'content' => $this->snipServiceFactory->create($snip)->get(),
        ]);
    }

    #[Route('/raw/{snip}', name: '_raw')]
    public function raw(Snip $snip, Request $request): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::VIEW, $snip);

        $response = new Response(
            $this->snipServiceFactory->create($snip)->get(),
            Response::HTTP_OK,
            ['Content-Type' => 'text/html']
        );
        $response
            ->setVary(['Accept', 'Accept-Encoding'])
            ->setEtag(md5($response->getContent()))
            ->setTtl(3600)
            ->setClientTtl(300);

        if (!$request->isNoCache()) {
            $response->isNotModified($request);
        }

        return $response;
//        return $this->render('snip/single.html.twig', [
//            'snip' => $snip,
//            'content' => $this->snipServiceFactory->create($snip)->get(),
//        ]);
    }

    #[Route('/edit/{snip}', name: '_edit')]
    public function edit(Snip $snip, Request $request): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::EDIT, $snip);

        $form = $this->createForm(SnipType::class, $snip);
        $form->add('Save', SubmitType::class);
        if ($snip->getId()) {
            $form->get('content')->setData($this->snipServiceFactory->create($snip)->get());
        }

        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $this->repository->save($snip);
            $this->snipServiceFactory->create($snip)->update($form->get('content')->getData());

            $this->addFlash('success', sprintf('Snip "%s" saved successfully', $snip));

            return $this->redirectToRoute('snip_single', [
                'snip' => $snip->getId(),
            ]);
        }

        return $this->render('snip/edit.html.twig', [
            'snip' => $snip,
            'form' => $form->createView(),
        ]);
    }

    #[Route('/new', name: '_new')]
    public function new(Request $request): Response
    {
        $snip = new Snip();
        $snip->setCreatedAtTodayNoSeconds()
            ->setCreatedBy($this->getUser());

        return $this->edit($snip, $request);
    }
}